#ifndef SQUASHFS_FS_SB
#define SQUASHFS_FS_SB
/*
 * Squashfs
 *
 * Copyright (c) 2002, 2003, 2004, 2005, 2006
 * Phillip Lougher <phillip@lougher.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * squashfs_fs_sb.h
 */

#include <linux/squashfs_fs.h>

/* including <linux/zlib.h> here breaks too much in other places of kernel code */
#ifndef IN_SQUASHFS
#  ifndef FAR
#    define FAR
#  endif
typedef unsigned char Bytef_;
typedef unsigned int  uInt_;
typedef unsigned long uLong_;

typedef struct z_stream_s_ {
    Bytef_   *next_in;  /* next input byte */
    uInt_    avail_in;  /* number of bytes available at next_in */
    uLong_   total_in;  /* total nb of input bytes read so far */

    Bytef_   *next_out; /* next output byte should be put there */
    uInt_    avail_out; /* remaining free space at next_out */
    uLong_   total_out; /* total nb of bytes output so far */

    char     *msg;      /* last error message, NULL if no error */
    struct internal_state *state; /* not visible by applications */

    void     *workspace; /* memory allocated for this stream */

    int     data_type;  /* best guess about the data type: ascii or binary */
    uLong_  adler;      /* adler32 value of the uncompressed data */
    uLong_  reserved;   /* reserved for future use */
} z_stream_;
#endif

struct squashfs_cache {
	long long	block;
	int		length;
	long long	next_index;
	char		*data;
};

struct squashfs_fragment_cache {
	long long	block;
	int		length;
	unsigned int	locked;
	char		*data;
};

struct squashfs_sb_info {
	struct squashfs_super_block	sblk;
	int			devblksize;
	int			devblksize_log2;
	int			swap;
	struct squashfs_cache	*block_cache;
	struct squashfs_fragment_cache	*fragment;
	int			next_cache;
	int			next_fragment;
	int			next_meta_index;
	unsigned int		*uid;
	unsigned int		*guid;
	long long		*fragment_index;
	unsigned int		*fragment_index_2;
	unsigned int		read_size;
	char			*read_data;
	char			*read_page;
	struct semaphore	read_data_mutex;
	struct semaphore	read_page_mutex;
	struct semaphore	block_cache_mutex;
	struct semaphore	fragment_mutex;
	struct semaphore	meta_index_mutex;
	wait_queue_head_t	waitq;
	wait_queue_head_t	fragment_wait_queue;
	struct meta_index	*meta_index;
#ifdef IN_SQUASHFS
	z_stream		stream;
#else
	z_stream_		stream;
#endif
	struct inode		*(*iget)(struct super_block *s,  squashfs_inode_t
				inode);
	long long		(*read_blocklist)(struct inode *inode, int
				index, int readahead_blks, char *block_list,
				unsigned short **block_p, unsigned int *bsize);
	int			(*read_fragment_index_table)(struct super_block *s);
};
#endif
